/* ---------------------------------------------------------------------------

Copyright (C) 1990, 1991, 1993, 1994, 2002.

A. Ronald Gallant
Post Office Box 659
Chapel Hill NC 27514-0659 
USA   

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, provided 
that the above copyright notice appears in all copies and that both that 
copyright notice and this permission notice appear in supporting 
documentation.  

This software is provided "as is" without any expressed or implied warranty.

----------------------------------------------------------------------------*/

#ifndef __FILE_MODEL_H_SEEN__

#include "status.h"

extern status s;

class model
{

private:
  REAL  nds[256];
  REAL  pop[256];
  REAL  nyse[256];
  REAL  dflat[256];

public:
  realmat e_t;      // e_t    is 1 by 1
  realmat dele_t;   // dele_t is 1 by 2
  realmat Z_t;      // Z_t    is 3 by 1

          model();
          ~model();

  int     initialize();
  int     terminate();

  void    e(INTEGER t);      
  void    dele(INTEGER t);   
  void    Z(INTEGER t);      

};

#define __FILE_MODEL_H_SEEN__ 
#endif

